from django.contrib import admin

from huscy.subject_notes.models import SubjectNote, SubjectNoteTag
from huscy.subject_notes.services import create_subject_note


class SubjectNoteAdmin(admin.ModelAdmin):

    date_hierarchy = 'created_at'
    fields = 'subject', 'note_tag', 'text', 'creator', 'created_at'
    list_display = 'subject', 'note_tag', 'text', 'creator', 'created_at'
    list_filter = 'note_tag',
    readonly_fields = 'creator', 'created_at'
    search_fields = 'subject__contact__display_name', 'subject__id'

    def save_model(self, request, note, form, change):
        create_subject_note(note.subject, request.user, note.note_tag, note.text)


admin.site.register(SubjectNote, SubjectNoteAdmin)
admin.site.register(SubjectNoteTag)



import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('subjects', '0015_alter_subject_options'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='SubjectNoteTag',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('text', models.CharField(max_length=255, verbose_name='Text')),
            ],
            options={
                'verbose_name': 'Subject note tag',
                'verbose_name_plural': 'Subject note tags',
                'ordering': ('text',),
            },
        ),
        migrations.CreateModel(
            name='SubjectNote',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('text', models.TextField(blank=True, verbose_name='Text')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='Created at')),
                ('creator', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL, verbose_name='Creator')),
                ('subject', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='notes', to='subjects.subject', verbose_name='Subject')),
                ('note_tag', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='+', to='subject_notes.subjectnotetag', verbose_name='Subject note tag')),
            ],
            options={
                'verbose_name': 'Subject note',
                'verbose_name_plural': 'Subject notes',
                'ordering': ('subject__contact__display_name', '-created_at'),
            },
        ),
    ]



from rest_framework import serializers

from huscy.subject_notes import models, services


class SubjectNoteSerializer(serializers.ModelSerializer):
    created_at = serializers.DateTimeField(read_only=True)
    creator = serializers.HiddenField(default=serializers.CurrentUserDefault())
    creator_username = serializers.CharField(source='creator.username', read_only=True)

    class Meta:
        model = models.SubjectNote
        fields = (
            'id',
            'created_at',
            'creator',
            'creator_username',
            'note_tag',
            'text',
        )

    def create(self, validated_data):
        return services.create_subject_note(**validated_data)


class SubjectNoteTagSerializer(serializers.ModelSerializer):

    class Meta:
        model = models.SubjectNoteTag
        fields = (
            'id',
            'text',
        )

    def create(self, validated_data):
        return services.create_subject_note_tag(**validated_data)



from rest_framework.permissions import BasePermission


class SubjectNotePermission(BasePermission):

    def has_permission(self, request, view):
        if request.user.has_perm('subjects.change_subject'):
            return True

        if request.method == 'POST':
            return request.user.has_perm('subject_notes.add_subjectnote')

        return False

    def has_object_permission(self, request, view, subject_note):
        if request.method == 'DELETE':
            if request.user.has_perm('subjects.change_subject'):
                return (request.user.has_perm('subject_notes.delete_subjectnote') or
                        request.user.has_perm('subject_notes.delete_subjectnote', subject_note))
        return True



from guardian.shortcuts import assign_perm

from huscy.subject_notes.models import SubjectNote, SubjectNoteTag


def create_subject_note(subject, creator, note_tag, text=''):
    args = dict(subject=subject, creator=creator, note_tag=note_tag)
    if note_tag.text.lower() == 'other':
        args.setdefault('text', text)

    note = SubjectNote.objects.create(**args)
    assign_perm('subject_notes.delete_subjectnote', creator, note)

    return note


def create_subject_note_tag(text):
    return SubjectNoteTag.objects.create(text=text)


def get_subject_notes(subject):
    return SubjectNote.objects.filter(subject=subject)


def get_subject_note_tags():
    return SubjectNoteTag.objects.all()



from django.shortcuts import get_object_or_404
from rest_framework.mixins import CreateModelMixin, DestroyModelMixin
from rest_framework.permissions import DjangoModelPermissions, IsAuthenticated
from rest_framework.viewsets import GenericViewSet, ModelViewSet

from huscy.subject_notes import services, serializer
from huscy.subject_notes.permissions import SubjectNotePermission
from huscy.subjects.models import Subject


class SubjectNoteViewSet(CreateModelMixin, DestroyModelMixin, GenericViewSet):
    permission_classes = IsAuthenticated, SubjectNotePermission
    serializer_class = serializer.SubjectNoteSerializer

    def initial(self, request, *args, **kwargs):
        super().initial(request, *args, **kwargs)
        self.subject = get_object_or_404(Subject, pk=self.kwargs['subject_pk'])

    def get_queryset(self):
        return services.get_subject_notes(self.subject)

    def perform_create(self, serializer):
        self.object = serializer.save(subject=self.subject)


class SubjectNoteTagViewSet(ModelViewSet):
    permission_classes = IsAuthenticated, DjangoModelPermissions
    serializer_class = serializer.SubjectNoteTagSerializer

    def get_queryset(self):
        return services.get_subject_note_tags()



from django.urls import include, path
from rest_framework.routers import DefaultRouter

from huscy.subject_notes.views import SubjectNoteViewSet, SubjectNoteTagViewSet
from huscy.subjects.urls import subject_router

router = DefaultRouter()
router.register('subjectnotetags', SubjectNoteTagViewSet, basename='subjectnotetag')

subject_router.register('notes', SubjectNoteViewSet, basename='subjectnote')


urlpatterns = [
    path('api/', include(router.urls)),
    path('api/', include(subject_router.urls)),
]



from django.conf import settings
from django.db import models
from django.utils.translation import gettext_lazy as _

from huscy.subjects.models import Subject


class SubjectNoteTag(models.Model):
    text = models.CharField(_('Text'), max_length=255)

    def __str__(self):
        return self.text

    class Meta:
        ordering = 'text',
        verbose_name = _('Subject note tag')
        verbose_name_plural = _('Subject note tags')


class SubjectNote(models.Model):
    subject = models.ForeignKey(Subject, on_delete=models.CASCADE, related_name='notes',
                                verbose_name=_('Subject'))

    note_tag = models.ForeignKey(SubjectNoteTag, on_delete=models.PROTECT, related_name='+',
                                 verbose_name=_('Subject note tag'))
    text = models.TextField(_('Text'), blank=True)

    creator = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.PROTECT,
                                verbose_name=_('Creator'))
    created_at = models.DateTimeField(_('Created at'), auto_now_add=True)

    class Meta:
        ordering = 'subject__contact__display_name', '-created_at'
        verbose_name = _('Subject note')
        verbose_name_plural = _('Subject notes')
